<?php

defined('_JEXEC') or die('Restricted access');

class CrossfitModelDashboard extends JModelList {

    private $db;

    public function __construct($config = array()) {
        parent::__construct($config);

        $this->db = JFactory::getDbo();
    }

    private function executeQueryNumber($query) {
        $this->db->setQuery($query);
        $this->db->execute();
        return $this->db->getNumRows();
    }

    public function getIscrizioni() {

        $query = $this->db->getQuery(true);
        $query->select("DISTINCT CXU.idUser")->from($this->db->quoteName('#__crossfit_users') . " CXU")->where("published = 1");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioniAttive() {
        $query = $this->db->getQuery(true);

        $query->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_iscrizioni') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->where("dataScadenza >= NOW()");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioniNonPagate() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->join('INNER', $this->db->quoteName('#__crossfit_memberships') . " M on M.idUser = CXU.idUser")
                ->where("CXU.published = 1 AND CURRENT_DATE BETWEEN M.dataInizio AND DATE_ADD(M.dataInizio, INTERVAL M.durata YEAR) AND M.pagato = 0");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioniScadute() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_iscrizioni') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->where("dataScadenza <= NOW() AND CXU.published = 1");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioniNonEffetuate() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->join('INNER', $this->db->quoteName('#__crossfit_memberships') . " M on M.idUser = CXU.idUser")
                ->where("CXU.published = 1 AND CURRENT_DATE BETWEEN M.dataInizio AND DATE_ADD(M.dataInizio, INTERVAL M.durata YEAR) AND M.pagato = 0");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioni7() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_iscrizioni') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->where("dataScadenza BETWEEN NOW() AND DATE_ADD(NOW(), INTERVAL 7 DAY) AND CXU.published = 1");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getIscrizioni15() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_iscrizioni') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->where("dataScadenza BETWEEN NOW() AND DATE_ADD(NOW(), INTERVAL 15 DAY) AND CXU.published = 1");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getCertificatiValidi() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->where("published = 1 AND NOW() <= DATE_ADD(dataCertificato, INTERVAL 1 YEAR)");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getCertificatiNonConsegnati() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->where("published = 1 AND (dataCertificato IS NULL OR dataCertificato = '0000-00-00')");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getCertificatiScaduti() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->where("published = 1 AND NOW() >= DATE_ADD(dataCertificato, INTERVAL 1 YEAR)");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getCertificati7() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->where("published = 1 AND DATE_ADD(dataCertificato, INTERVAL 1 YEAR) BETWEEN DATE_ADD(NOW(), INTERVAL 0 DAY) AND DATE_ADD(NOW(), INTERVAL 7 DAY)");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getCertificati15() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_users') . " CXU")
                ->where("published = 1 AND (DATE_ADD(dataCertificato, INTERVAL 1 YEAR) BETWEEN DATE_ADD(NOW(), INTERVAL 7 DAY) AND DATE_ADD(NOW(), INTERVAL 15 DAY))");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getAbbonamenti() {
        $query = $this->db->getQuery(true);

        $query->select("DISTINCT CXU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_abbonamenti') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getAbbonamentiAttivi() {
        $query = $this->db->getQuery(true);

        $query->select("DISTINCT SU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_abbonamenti') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where('CXU.idUser IN (SELECT idUser FROM #__crossfit_v_scadenze_abbonamenti WHERE NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento AND sospeso = 0)')
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getAbbonamentiScaduti() {
        $query = $this->db->getQuery(true);

        $query
                ->select("DISTINCT SU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_abbonamenti') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where('CXU.idUser IN (SELECT idUser FROM #__crossfit_v_scadenze_abbonamenti WHERE dataScadenzaAbbonamento < NOW() AND sospeso = 0)')
                ->where('CXU.idUser NOT IN (SELECT idUser FROM #__crossfit_v_scadenze_abbonamenti WHERE NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento)')
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    public function getAbbonamentiSospesi() {
        $query = $this->db->getQuery(true);

        $query->select("DISTINCT SU.idUser")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_abbonamenti') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where('CXU.idUser IN (SELECT idUser FROM #__crossfit_v_scadenze_abbonamenti WHERE NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento AND sospeso = 1)')
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        return $this->executeQueryNumber($query);
    }

    private function getAbbonamentiInScadenza($num = 7) {
        $query = $this->db->getQuery(true);

        switch ($num) {
            case 7:
                $where = "dataScadenzaAbbonamento BETWEEN CURRENT_DATE AND DATE_ADD(CURRENT_DATE, INTERVAL 7 DAY)";
                break;
            case 15:
                $where = "dataScadenzaAbbonamento BETWEEN DATE_ADD(CURRENT_DATE, INTERVAL 8 DAY) AND DATE_ADD(CURRENT_DATE, INTERVAL 15 DAY)";
                break;
            case 30:
                $where = "dataScadenzaAbbonamento BETWEEN DATE_ADD(CURRENT_DATE, INTERVAL 16 DAY) AND DATE_ADD(CURRENT_DATE, INTERVAL 30 DAY)";
                break;
            case 60:
            default:
                $where = "dataScadenzaAbbonamento BETWEEN DATE_ADD(CURRENT_DATE, INTERVAL 31 DAY) AND DATE_ADD(CURRENT_DATE, INTERVAL 60 DAY)";
                break;
        }

        $query
                ->select("COUNT(*)")
                ->from($this->db->quoteName('#__crossfit_v_scadenze_abbonamenti') . " SU")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON SU.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where($where)
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        return $this->db->setQuery($query)->loadResult();
    }

    public function getScadenza60() {
        return $this->getAbbonamentiInScadenza(60);
    }

    public function getScadenza30() {
        return $this->getAbbonamentiInScadenza(30);
    }

    public function getScadenza15() {
        return $this->getAbbonamentiInScadenza(15);
    }

    public function getScadenza7() {
        return $this->getAbbonamentiInScadenza(7);
    }

    public function getIDSSubscriptions() {
        return array(
            'kids' => $this->db->setQuery("SELECT idSubscription FROM #__crossfit_subscriptions WHERE LOWER(nome) = 'kids'")->loadResult(),
            'teen' => $this->db->setQuery("SELECT idSubscription FROM #__crossfit_subscriptions WHERE LOWER(nome) = 'teen'")->loadResult(),
            'elements' => $this->db->setQuery("SELECT idSubscription FROM #__crossfit_subscriptions WHERE LOWER(nome) = 'elements'")->loadResult()
        );
    }

    public function getTipologieAbbonamento() {

        $return = array();

        $query = $this->db->getQuery(true);

        $IDSSubscription = $this->getIDSSubscriptions();

        $idKids = $IDSSubscription['kids'];
        $idTeen = $IDSSubscription['teen'];
        $idElements = $IDSSubscription['elements'];

        $query->select("mesi, COUNT(*) as count")
                ->from("#__crossfit_v_scadenze_abbonamenti as US")
                ->innerJoin("#__crossfit_subscriptions as SU on US.idSubscription = SU.idSubscription")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON US.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where("mesi > 0")
                ->where("US.sospeso = 0")
                ->where("NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento")
                ->group("mesi")
                ->order("mesi")
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        $this->db->setQuery($query);

        $objects = $this->db->loadObjectList();
        if ($objects) {
            foreach ($objects as $obj) {
                $return['mensili'][$obj->mesi] = $obj->count;
            }
        }

        $query->clear();

        $query->select("COUNT(*) as count")
                ->from("#__crossfit_v_scadenze_abbonamenti as US")
                ->innerJoin("#__crossfit_subscriptions as SU on US.idSubscription = SU.idSubscription")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON US.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where("NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento")
                ->where("SU.idSubscription = $idKids")
                ->where("US.sospeso = 0")
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        $this->db->setQuery($query);

        $return['kids'] = $this->db->loadResult();

        $query->clear();

        $query->select("COUNT(*) as count")
                ->from("#__crossfit_v_scadenze_abbonamenti as US")
                ->innerJoin("#__crossfit_subscriptions as SU on US.idSubscription = SU.idSubscription")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON US.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where("NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento")
                ->where("SU.idSubscription = $idTeen")
                ->where("US.sospeso = 0")
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        $this->db->setQuery($query);

        $return['teen'] = $this->db->loadResult();

        $query->clear();

        $query->select("COUNT(*) as count")
                ->from("#__crossfit_v_scadenze_abbonamenti as US")
                ->innerJoin("#__crossfit_subscriptions as SU on US.idSubscription = SU.idSubscription")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON US.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where("NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento")
                ->where("US.idSubscription = $idElements")
                ->where("US.sospeso = 0")
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        $this->db->setQuery($query);

        $return['elements'] = $this->db->loadResult();

        $query->clear();

        $query->select("COUNT(*) as count")
                ->from("#__crossfit_v_scadenze_abbonamenti as US")
                ->innerJoin("#__crossfit_subscriptions as SU on US.idSubscription = SU.idSubscription")
                ->innerJoin($this->db->quoteName('#__crossfit_users') . " CXU ON US.idUser = CXU.idUser")
                ->innerJoin($this->db->quoteName('#__users') . " JU ON JU.id = CXU.idUserJoomla")
                ->where("NOW() BETWEEN dataInizio AND dataScadenzaAbbonamento")
                ->where("adIngressiTotali = 1")
                ->where("US.sospeso = 0")
                ->where("CXU.published = 1")
                ->where("JU.block = 0");

        $query = $this->checkLocation($query);

        $this->db->setQuery($query);

        $return['ingressiTotali'] = $this->db->loadResult();
        return $return;
    }

    public function checkLocation($query) {
        $idPalestra = CrossfitHelper::getActualIdPalestra();

        if (CrossfitHelper::isConsultazione()) {
            $palestreConsultazione = CrossfitHelper::getPalestreConsultazione();
            foreach ($palestreConsultazione as $palestra) {
                $ids[] = $palestra->id;
            }
            if ($ids) {
                $query->where("CXU.idPalestra IN (" . implode(", ", $ids) . ")");
            }
        } elseif ($idPalestra) {
            $query->where("CXU.idPalestra = " . $idPalestra);
        }

        return $query;
    }

    public function getSubscriptionsSales() {

        $query = $this->db->getQuery(true);
        $query->select("MONTH(SU.dataInizio) as month, YEAR(SU.dataInizio) as year, SUM(SU.pagato) as summ")
                ->from("#__crossfit_subscriptions_users as SU")
                ->innerJoin("#__crossfit_users as CXU ON CXU.idUser = SU.idUser")
                ->where("YEAR(SU.dataInizio) >=" . $this->db->quote(date("Y") - 1))
                ->group("YEAR(SU.dataInizio), MONTH(SU.dataInizio)");

        $query = $this->checkLocation($query);

        $objects = $this->db->setQuery($query)->loadObjectList();
        $return = array();
        if ($objects) {
            foreach ($objects as $ob) {
                $return[$ob->month][$ob->year] = $ob->summ;
            }
        }

        return $return;
    }

}
